<?php
// $Id$

/**
 * @file
 * Class definition of FeedsFeedNodeProcessorWithGC.
 * 
 * @author Wayne Eaker <wayne@zengenuity.com>
 */

/**
 * Changes FeedsFeedNodeProcessor to add garbage collection of items no 
 * longer in the feed.
 */
class FeedsNodeProcessorWithGC extends FeedsNodeProcessor {
  
  /**
   * Implementation of FeedsProcessor::process().
   * 
   * This function has been copied from FeedsNodeProcessor, with a few changes to call 
   * garbage collection. (Changes are marked with comments labeled "GC".)
   */
  public function process(FeedsImportBatch $batch, FeedsSource $source) {
    
    //GC: Keep track of all nids in this feed.
    $nids_in_feed = array();
    
    // Keep track of processed items in this pass.
    $processed = 0;

    while ($item = $batch->shiftItem()) {
      
      //GC: If there's an existing nid, store that.
      //$nid = $this->existingItemId($item, $source);
      $nid = $this->existingItemId($batch, $source);
      if($nid) {
        $nids_in_feed[] = $nid;
      }
      // Create/update if item does not exist or update existing is enabled.
      if (!$nid || $this->config['update_existing']) {

        $node = new stdClass();
        $hash = $this->hash($item);

        if (!empty($nid)) {

          // If hash of this item is same as existing hash there is no actual
          // change, skip.
          if ($hash == $this->getHash($nid)) {
            continue;
          }

          // If updating populate nid and vid avoiding an expensive node_load().
          $node->nid = $nid;
          $node->vid = db_result(db_query("SELECT vid FROM {node} WHERE nid = %d", $nid));
          $batch->updated++;
        }
        else {
          $batch->created++;
        }

        // Populate and prepare node object.
        $node->type = $this->config['content_type'];
        $node->created = FEEDS_REQUEST_TIME;
        $node->feeds_node_item = new stdClass();
        $node->feeds_node_item->hash = $hash;
        $node->feeds_node_item->id = $this->id;
        $node->feeds_node_item->imported = FEEDS_REQUEST_TIME;
        $node->feeds_node_item->feed_nid = $source->feed_nid;
        static $included;
        if (!$included) {
          module_load_include('inc', 'node', 'node.pages');
          $included = TRUE;
        }
        node_object_prepare($node);

        // Populate properties that are set by node_object_prepare().
        $node->log = 'Created/updated by FeedsNodeProcessor';
        $node->uid = 0;

        // Execute mappings from $item to $node.
        //$this->map($item, $node);
		$this->map($batch, $node);
        // Save the node.
        node_save($node);
        
        //GC: Store any new nids created.
        if(!in_array($node->nid, $nids_in_feed)) {
          $nids_in_feed[] = $node->nid;
        }
        
      }

      $processed++;
      if ($processed >= variable_get('feeds_node_batch_size', FEEDS_NODE_BATCH_SIZE)) {
        return (1.0 / ($batch->total + 1)) * ($batch->updated + $batch->created); // Add + 1 to make sure that result is not 1.0 = finished.
      }
    }
    
    //GC: Call garbage collector.
    $garbage_counts = $this->garbage($source->feed_nid, $nids_in_feed);

    // Set messages.
    if ($batch->created) {
      drupal_set_message(t('Created !number !type nodes.', array('!number' => $batch->created, '!type' => node_get_types('name', $this->config['content_type']))));
    }
    elseif ($batch->updated) {
      drupal_set_message(t('Updated !number !type nodes.', array('!number' => $batch->updated, '!type' => node_get_types('name', $this->config['content_type']))));
    }
    else {
      drupal_set_message(t('There is no new content.'));
    }
    
    //GC: Output garbage colleciton messages
    if($garbage_counts['unpublished']) {
      drupal_set_message(t('Unpublished !number !type nodes.', array('!number' => $garbage_counts['unpublished'], '!type' => node_get_types('name', $this->config['content_type']))));
    }
    if($garbage_counts['published']) {
      drupal_set_message(t('Republished !number !type nodes.', array('!number' => $garbage_counts['published'], '!type' => node_get_types('name', $this->config['content_type']))));
    }

    return FEEDS_BATCH_COMPLETE;  
    
  }
  
  /**
   * Collects garbage
   */
  public function garbage($feed_nid, $nids_in_feed) {
    $counts = array('unpublished' => 0, 'published' => 0);
    
    //Get all published items from this feed that are not in this feed.
    $sql = "SELECT n.nid FROM {feeds_node_item} fni LEFT JOIN {node} n ON fni.nid = n.nid WHERE fni.feed_nid = %d AND n.status = 1";
    if(!empty($nids_in_feed)) {
      $sql .= " AND n.nid NOT IN (" . db_placeholders($nids_in_feed, 'int') . ")";
    }
    $result = db_query($sql, array_merge(array($feed_nid), $nids_in_feed));
    
    while($node = db_fetch_object($result)) {
      //Unpublish and save.
      $node = node_load(array('nid' => $node->nid));
      $node->status = 0; 
      node_save($node);
      $counts['unpublished']++;
    }
    
    //Get all unpublished items that may be back in the feed.
    $sql = "SELECT n.nid FROM {feeds_node_item} fni LEFT JOIN {node} n ON fni.nid = n.nid WHERE fni.feed_nid = %d AND n.status = 0";
    if(!empty($nids_in_feed)) {
      $sql .= " AND n.nid IN (" . db_placeholders($nids_in_feed, 'int') . ")";
    }
    $result = db_query($sql, array_merge(array($feed_nid), $nids_in_feed));
    
    while($node = db_fetch_object($result)) {
      //Republish and save.
      $node = node_load(array('nid' => $node->nid));
      $node->status = 1; 
      node_save($node);
      $counts['published']++;
    }
    
    return $counts;
  }
  
}


